import 'package:flutter/material.dart';
import 'package:flutter_gadgets/flutter_gadgets.dart';

import '../control/game_screen_control.dart';
import '../model/game.dart';

class GameScreenVM {
  TextEditingController controller;
  String errorMessage;
  bool canPlay = true;
}

class GameScreen extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return ViewModelGadget<GameScreenVM>(
      GameScreenVM(),
      initStateCallback: (final viewModel) {
        viewModel.controller = TextEditingController();
      },
      disposeCallback: (final viewModel) {
        viewModel.controller.dispose();
      },
      child: _child(context),
    );
  }

  Widget _child(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('Game'),
        centerTitle: false,
        actions: [
          IconButton(
            icon: Icon(Icons.refresh),
            onPressed: onResetPressed,
          )
        ],
      ),
      body: _body,
      floatingActionButton: SubscriberGadget<GameScreenVM>(
        builder: (_, viewModel) => FloatingActionButton(
          onPressed: viewModel.canPlay ? onNextTurnPressed : null,
          child: Icon(Icons.arrow_right_alt),
        ),
      ),
    );
  }

  Widget get _body => Padding(
        padding: const EdgeInsets.all(8.0),
        child: Column(
          children: [
            SubscriberGadget<Game>(
              builder: (_, game) => Text(game.hint),
            ),
            SizedBox(height: 16),
            Row(
              mainAxisSize: MainAxisSize.max,
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Text('Attempts:'),
                SubscriberGadget<Game>(
                  builder: (_, game) => Text(
                    game.attempts.toString(),
                  ),
                ),
              ],
            ),
            SizedBox(height: 32),
            SubscriberGadget<GameScreenVM>(
              builder: (_, viewModel) => TextField(
                controller: viewModel.controller,
                enabled: viewModel.canPlay,
                keyboardType: TextInputType.number,
                decoration: InputDecoration(
                  hintText: 'Insert a number',
                  errorText: viewModel.errorMessage,
                ),
              ),
            )
          ],
        ),
      );
}
