import 'dart:math' as math;

import 'package:flutter/material.dart';
import 'package:flutter_gadgets/flutter_gadgets.dart';

import '../const.dart';
import '../control/home_screen_control.dart';
import '../model/match.dart';

class HomeScreen extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: _appBar,
      body: _body(context),
    );
  }

  AppBar get _appBar {
    return AppBar(
      title: Text('Rock Paper Scissor'),
      centerTitle: false,
      actions: [
        IconButton(
          icon: Icon(Icons.refresh),
          onPressed: onRefreshPressed,
        )
      ],
    );
  }

  Widget _body(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: SubscriberGadget<Match>(
        builder: (_, Match match) => Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            const Text(
              'Play a match!',
              style: TextStyle(
                fontWeight: FontWeight.w700,
              ),
            ),
            Text(match.matchStateDescription),
            Text(match.lastTurnStateDescription),
            const SizedBox(height: 16),
            RowText('Player wins:', match.playerWins.toString()),
            RowText('Ties:', match.ties.toString()),
            RowText('Cpu wins:', match.cpuWins.toString()),
            const SizedBox(height: 16),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceAround,
              children: [
                TextButton(
                  'ROCK',
                  () => onButtonPressed(HandSign.rock),
                  match.matchState == MatchState.playing,
                ),
                TextButton(
                  'PAPER',
                  () => onButtonPressed(HandSign.paper),
                  match.matchState == MatchState.playing,
                ),
                TextButton(
                  'SCISSOR',
                  () => onButtonPressed(HandSign.scissor),
                  match.matchState == MatchState.playing,
                ),
              ],
            ),
            const SizedBox(height: 16),
            HandSignRow()
          ],
        ),
      ),
    );
  }
}

class RowText extends StatelessWidget {
  RowText(this.firstText, this.secondText);

  final String firstText;
  final String secondText;

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        Text(firstText),
        Text(
          secondText,
          style: TextStyle(fontWeight: FontWeight.bold),
        ),
      ],
    );
  }
}

class TextButton extends StatelessWidget {
  TextButton(this.text, this.onPressed, this.enabled);

  final String text;
  final VoidCallback onPressed;
  final bool enabled;

  @override
  Widget build(BuildContext context) {
    return MaterialButton(
      color: Theme.of(context).primaryColor,
      disabledColor: Colors.grey,
      onPressed: enabled ? onPressed : null,
      child: Text(
        text,
        style: TextStyle(color: Colors.white),
      ),
    );
  }
}

class HandSignRow extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        _playerImage,
        _cpuImage,
      ],
    );
  }

  Widget get _playerImage {
    return SubscriberGadget<Match>(
      builder: (_, Match match) {
        return _image(match.lastTurn?.playerChoice);
      },
    );
  }

  Widget get _cpuImage {
    return Transform(
      alignment: Alignment.center,
      transform: Matrix4.rotationY(math.pi),
      child: SubscriberGadget<Match>(
        builder: (_, Match match) {
          return _image(match.lastTurn?.cpuChoice);
        },
      ),
    );
  }

  Widget _image(HandSign sign) {
    if (sign == HandSign.paper) return Image.asset(IMAGE_PATH_PAPER);
    if (sign == HandSign.rock) return Image.asset(IMAGE_PATH_ROCK);
    return Image.asset(IMAGE_PATH_SCISSOR);
  }
}
