import 'package:flutter/material.dart';

import 'model_gadget.dart';
import 'observable_model.dart';
import 'service_container.dart';

/// Root widget for a Flutter Gadgets application.
/// Initializes the [ServiceContainer] using the given services and
/// creates an [ObservableModel] that holds the given beans.
///
/// It is used as follows:
///
/// ```dart
/// void main() => runApp(AppGadget(
///   child: CustomChild,
///   services: {CustomService: CustomService()},
///   beans: {toBeanKey(MyBean): MyBean()}
/// ));
/// ```
class AppGadget extends StatelessWidget {
  AppGadget({
    @required this.child,
    Map<Type, Object> services,
    Map<String, Object> beans,
  }) {
    final observableModel = ObservableModel();
    if (beans != null) observableModel.putAllBeans(beans);
    Map<Type, Object> serviceMap = Map<Type, Object>();
    serviceMap[ObservableModel] = observableModel;
    if (services != null) serviceMap.addAll(services);
    ServiceContainer().init(serviceMap);
  }

  /// The widget tree wrapped by [AppGadget].
  final Widget child;

  @override
  Widget build(BuildContext context) {
    return _ModelListener(ServiceContainer().observableModel, child);
  }
}

class _ModelListener extends StatefulWidget {
  _ModelListener(this.observableModel, this.child);

  final ObservableModel observableModel;
  final Widget child;

  @override
  _ModelListenerState createState() => _ModelListenerState();
}

class _ModelListenerState extends State<_ModelListener> {
  Set<String> _updatedProperties = Set<String>();

  @override
  void initState() {
    super.initState();
    final observableModel = widget.observableModel;
    observableModel.addListener(_modelListener);
  }

  @override
  void dispose() {
    final observableModel = widget.observableModel;
    observableModel.removeListener(_modelListener);
    ServiceContainer().clear();
    super.dispose();
  }

  void _modelListener() {
    setState(() {
      _updatedProperties = Set<String>();
      _updatedProperties.addAll(widget.observableModel.updatedProperties);
    });
  }

  @override
  Widget build(BuildContext context) {
    widget.observableModel.clearUpdatedProperties();
    return ModelGadget(
      child: widget.child,
      updatedProperties: _updatedProperties,
    );
  }
}
