import 'observable_model.dart';

/// A singleton used to store the services map.
class ServiceContainer {
  factory ServiceContainer() => instance;

  ServiceContainer._private();

  static final instance = ServiceContainer._private();

  final Map<Type, Object> _services = Map<Type, Object>();
  bool _initialized = false;

  /// Initializes the services map with the given services.
  /// Called by [AppGadget] during initialization.
  ///
  /// NOTE: Do not call this method manually!
  init(Map<Type, Object> services) {
    if (_initialized) return;
    _initialized = true;
    _services.addAll(services);
  }

  /// Retrieves a service from the services map.
  T get<T>() => _services[T] as T;

  /// Clears the services map and restores the internal initialization state.
  void clear() {
    _services.clear();
    _initialized = false;
  }

  /// Shortcut for retrieving the [ObservableModel].
  ObservableModel get observableModel =>
      _services[ObservableModel] as ObservableModel;
}
