import 'package:flutter/material.dart';

import 'model_gadget.dart';
import 'observable_model.dart';
import 'service_container.dart';

/// A builder function that exposes a value retrieved from the [ObservableModel].
typedef Widget SubscriberGadgetBuilder<T>(BuildContext context, T value);

/// A widget that observes a bean.
/// A bean can be identified using its type and an optional token.
/// It can observe single or multiple properties of a bean.
class SubscriberGadget<T> extends StatelessWidget {
  SubscriberGadget({this.token, this.properties, @required this.builder});

  /// An optional token used to identify a bean.
  final String token;

  /// The optional list of observed properties.
  final List<String> properties;

  /// The builder function.
  final SubscriberGadgetBuilder<T> builder;

  void _inheritFromAspects(BuildContext context) {
    if (properties == null || properties.isEmpty) {
      InheritedModel.inheritFrom<ModelGadget>(
        context,
        aspect: toBeanKey(T, token: token),
      );
      return;
    }
    properties.forEach(
      (String property) => InheritedModel.inheritFrom<ModelGadget>(
        context,
        aspect: toBeanKey(T, token: token, property: property),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    _inheritFromAspects(context);
    return this.builder(
      context,
      ServiceContainer().observableModel.getBean<T>(token),
    );
  }
}
